//@+leo-ver=4
//@+node:@file host/lud-pnx8550.c
//@@language c
/* Copyright Philips Research Labs, Redhill, UK. 2001
 *
 * Prototype Software
 *
 * Contact: Tim Everett (tim.everett@philips.com)
 *
 */

#define MODULE

#ifndef MODULE
#error This code must be defined as a kernel module
#endif


/***** Includes *****/
//@<< lud-pnx8550 #includes >>
//@+node:<< lud-pnx8550 #includes >>
#ifdef CONFIG_SMP
#define __SMP__
#endif

#include <asm/segment.h>
#include <asm/io.h>
#include <asm/page.h>
#include <asm/system.h>
#include <asm/uaccess.h>

#include <linux/module.h>
#include <linux/version.h>
#include <linux/types.h>
#include <linux/kernel.h>
#include <linux/fs.h>
#include <linux/mm.h>
#include <linux/errno.h>
#include <linux/pci.h>
#include <linux/ioport.h>
#include <linux/sched.h>
#include <linux/slab.h>
#include <linux/interrupt.h>
#include <linux/delay.h>
#include <linux/proc_fs.h>
#include <linux/vmalloc.h>
#include <linux/kmod.h>
#if LINUX_VERSION_CODE > 0x020600
#include <linux/cdev.h>
#include <linux/platform_device.h>
#endif


#include <stdarg.h>

#include "tmhw.h"

#include "compat.h"
#include "lud_mmio.h"
#include "luddite-host.h"
//#include "lud_type.h"
#define  LUD_TYPE_8550 3

#include "core.h"
#include "trimedia.h"

//@nonl
//@-node:<< lud-pnx8550 #includes >>
//@nl
//@<< lud-pnx8550 declarations >>
//@+node:<< lud-pnx8550 declarations >>


/* 2005-05-10 Wouter Batelaan (WB) Ideally we would
   do this: #include <$PRJROOT/src/mptk/mptk_root/inc/cfg/tmSysCfg.h>
   and calculate the right values, but that will be done later.
 */
#define VIPER_TOTAL_RAM     0x8000000 /* 128M */

/* 20050902 NAB: These are just defaults, and can be overridden with module params */
#define VIPER_MIPS_SDRAM_SIZE   0x4000000 /* 64M */
#define VIPER_TM0_SDRAM_SIZE    0x2000000 /* 32M */
#define VIPER_TM1_SDRAM_SIZE    0x0800000 /* 8M */

#define NDK_BIS_SIZE        0x1000

#define MMIO_BASE       0xbbe00000

#define VIPER_TM_MMIO_BASE  (((unsigned long)MMIO_BASE) - 0xA0000000UL) /*0x1BE00000 */
/* has to be set up here, as the tmlib stuff redefines MMIO_BASE!!!! */
static unsigned int mmio = VIPER_TM_MMIO_BASE;

#define VIPER_TM_MMIO_SIZE  0x20000
#define VIPER_TM1_IRQ       10
#define VIPER_TM2_IRQ       11
#define VIPER_TM_DSP_OFFSET 0x00100000
#define TM_CACHE_ROW_SIZE   64

#define VIPER_TM_APERTURE_START 0x10000000
#define VIPER_TM_APERTURE_END   0x12000000

/* 20050902 NAB - added mips_size and renamed sdram1 & sdram2 */
static int ram_size = VIPER_TOTAL_RAM;
static int mips_size = VIPER_MIPS_SDRAM_SIZE ;
static int tm0_size = VIPER_TM0_SDRAM_SIZE ;
static int tm1_size = VIPER_TM1_SDRAM_SIZE ;
static int sharedcached_size = 0;
static int tm_aperture_start = VIPER_TM_APERTURE_START ;
static int tm_aperture_end = VIPER_TM_APERTURE_END ;
static int freq = 240000000;
static int ndkbis = 0;

#if LINUX_VERSION_CODE < 0x020600
MODULE_PARM(freq,"i");
MODULE_PARM(ram_size,"i");
MODULE_PARM(mips_size,"i");
MODULE_PARM(tm0_size,"i");
MODULE_PARM(tm1_size,"i");
MODULE_PARM(sharedcached_size,"i");
MODULE_PARM(tm_aperture_start,"i");
MODULE_PARM(tm_aperture_end,"i");
MODULE_PARM(mmio,"i");
MODULE_PARM(ndkbis,"i");
#else
module_param(freq,int,S_IRUGO);
module_param(ram_size,int,S_IRUGO);
module_param(mips_size,int,S_IRUGO);
module_param(tm0_size,int,S_IRUGO);
module_param(tm1_size,int,S_IRUGO);
module_param(sharedcached_size,int,S_IRUGO);
module_param(tm_aperture_start,int,S_IRUGO);
module_param(tm_aperture_end,int,S_IRUGO);
module_param(mmio,int,S_IRUGO);
module_param(ndkbis,int,S_IRUGO);
#endif

// from tmman tmhal.c:
    // GRR 20030422:  Semi-secret, relatively undocumented info from Greg
    //   Lydon:  in multi-CPU NDK setup, MML wants last 128 KB (0x20000) of
    //   shared cached memory actually to be uncached(!!) for its data
    //   structures.  (See TMML_CACHE_CONTROL_SIZE, "TMML non-cached control
    //   struct," in, e.g.,
    //       boards/comps/tmbslVpci/src/tmbslVpci.c
    //       boards/comps/tmbslVpci/src/tm_psos/tmbslBoardOsSpecific.c
    //       boards/comps/tmbslV2Pci/src/tmbslV2Pci.c
    //       boards/comps/tmbslV2Pci/src/tm_psos/tmbslBoardOsSpecific.c )
    //   Hence in a multi-CPU setup (only), MMIO CLIMIT should be set to
    //   0x20000 less than what BIS ShNC key says, i.e., 0x3fa0000 if BIS
    //   key #1 says 0x3fc0000.  Ugly...  (Also note that ShCa key may be
    //   set to zero or not exist, at least in standalone TM systems.  So
    //   use ShNC key, which always exists, just to be safe.)
#define EXTRA_NC 0x20000


MODULE_LICENSE("GPL");
MODULE_AUTHOR("Philips Research");
MODULE_DESCRIPTION("Luddite PNX8550 driver");
//@-node:<< lud-pnx8550 declarations >>
//@nl
//@+others
//@+node:viper_int_handler1

#if LINUX_VERSION_CODE < 0x020600
static void
#else
static int
#endif
viper_int_handler1(int intNo,void* devId,struct pt_regs * useless)
{
        LUD_DEV* luddev = (LUD_DEV*)devId;
	unsigned int reg;

	reg = MMIOread(luddev,0x3e408);
	MMIOwrite(reg | 0x20000000,luddev,0x3e408);
	/* NOTE!!! cannot share interrupt... need code to check that this
           was really meant for us!! */
	lud_int_handler(luddev);
#if LINUX_VERSION_CODE >= 0x020600
	return IRQ_HANDLED;
#endif
}
//@-node:viper_int_handler1
//@+node:viper_int_handler2

#if LINUX_VERSION_CODE < 0x020600
static void
#else
static int
#endif
viper_int_handler2(int intNo,void* devId,struct pt_regs * useless)
{
        LUD_DEV* luddev = (LUD_DEV*)devId;
	unsigned int reg;

	reg = MMIOread(luddev,0x3e40c);
	MMIOwrite(reg | 0x20000000,luddev,0x3e40c);

	/* NOTE!!! cannot share interrupt... need code to check that this
           was really meant for us!! */
	lud_int_handler(luddev);

#if LINUX_VERSION_CODE >= 0x020600
        return IRQ_HANDLED;
#endif
}
//@-node:viper_int_handler2
//@+node:remap_pci_mem

#ifdef KSEG1
#undef KSEG1
#endif
#define KSEG1 0xA0000000UL
static unsigned long remap_pci_mem(unsigned long base, unsigned long size)
{
  return(base + KSEG1);
}
//@-node:remap_pci_mem
//@+node:viper_init

static void viper_init(LUD_DEV* dev)
{
	ptmhwDspRegs_t tmDsp =
		(ptmhwDspRegs_t)(dev->mmio_log+dev->mmio_offset+VIPER_TM_DSP_OFFSET);
	u32 dspCtrl = (readl(&tmDsp->control) & ~TMHW_DSP_CTL_ACTION_MASK)
			 | TMHW_DSP_CTL_ACTION_STOP_AND_RESET;
	u32 dramLo = mips_size; /* 20050902 NAB */
	/* 2005-05-10 Wouter Batelaan (WB) */
	u32 dramHi = ram_size;
        u32 dramCacheLim = mips_size + tm0_size + tm1_size + sharedcached_size - EXTRA_NC;

	writel(dspCtrl, (IOTYPE)&tmDsp->control);
	writel(dramHi, (IOTYPE)&tmDsp->dramHi);
	writel(dramLo, (IOTYPE)&tmDsp->dramLo);
	writel(dramCacheLim, (IOTYPE)&tmDsp->dramCacheLim);
	writel(tm_aperture_start, (IOTYPE)&tmDsp->apert1Lo);
	writel(tm_aperture_end, (IOTYPE)&tmDsp->apert1Hi);
	writel((u32)dev->sdram_phy, (IOTYPE)&tmDsp->startAddr);

//      MMIO_TMwrite(0,dev,dev->target_comm_reg);
//      MMIO_TMwrite(0,dev,dev->host_comm_reg);
}
//@-node:viper_init
//@+node:viper_start

static void viper_start(LUD_DEV* dev)
{
	ptmhwDspRegs_t tmDsp =
		(ptmhwDspRegs_t)(dev->mmio_log+dev->mmio_offset+VIPER_TM_DSP_OFFSET);
	u32 dspCtrl ;
	u32 dramLo = mips_size; /* 20050902 NAB */
	/* 2005-05-10 Wouter Batelaan (WB) */
	u32 dramHi = ram_size;
        u32 dramCacheLim = mips_size + tm0_size + tm1_size + sharedcached_size - EXTRA_NC;

	if (ndkbis) {
		if (dev->ndkbis) {
			printk("Clearing NDK BIS region (%p..%p)\n",
				(void*)dev->ndkbis, (void*)(dev->ndkbis+NDK_BIS_SIZE));
			memset_io((void*)dev->ndkbis, 0, NDK_BIS_SIZE);
		}
		else {
			printk("NDK BIS region not configured correctly?\n");
		}
	}
        writel(dramHi, (IOTYPE)&tmDsp->dramHi);
        writel(dramLo, (IOTYPE)&tmDsp->dramLo);
	writel(dramCacheLim, (IOTYPE)&tmDsp->dramCacheLim);
        writel(tm_aperture_start, (IOTYPE)&tmDsp->apert1Lo);
        writel(tm_aperture_end, (IOTYPE)&tmDsp->apert1Hi);
	writel((u32)dev->sdram_phy, (IOTYPE)&tmDsp->startAddr);

	dspCtrl = (readl((IOTYPE)&tmDsp->control) & ~TMHW_DSP_CTL_ACTION_MASK) |
                         TMHW_DSP_CTL_ACTION_START;
	writel(dspCtrl, (IOTYPE)&tmDsp->control);
}
//@-node:viper_start
//@+node:viper_reset

static void viper_reset(LUD_DEV* dev)
{
	ptmhwDspRegs_t tmDsp =
		(ptmhwDspRegs_t)(dev->mmio_log+dev->mmio_offset+VIPER_TM_DSP_OFFSET);
	u32 dspCtrl = (readl((IOTYPE)&tmDsp->control) & ~TMHW_DSP_CTL_ACTION_MASK) |
                         TMHW_DSP_CTL_ACTION_STOP_AND_RESET;
	writel(dspCtrl, (IOTYPE)&tmDsp->control);
}
//@-node:viper_reset
//@+node:decodePLL

static LUD_DEV  viper_desc1 = {
	"pnx8550",
	LUD_TYPE_8550,
	0,
	0,
	TM_INTERRUPT_NUM,
	viper_init,
	viper_start,
	viper_reset,
	tm_send_command,
	tm_send_request,
	tm_send_ack,
	0,
};

static LUD_DEV  viper_desc2 = {
	"pnx8550",
	LUD_TYPE_8550,
	0,
	0,
	TM_INTERRUPT_NUM,
	viper_init,
	viper_start,
	viper_reset,
	tm_send_command,
	tm_send_request,
	tm_send_ack,
	0,
};

static unsigned int decodePLL(unsigned int pll)
{
	unsigned int P =(pll>>2)&3;
	unsigned int M = (pll>>8)&0x3f;
	unsigned int N = (pll>>16)&0x1ff;
	unsigned int denom = 1;
	unsigned int ret;

	switch (P) {
	case 0: denom = 1; break;
	case 1: denom = 2; break;
	case 2: denom = 4; break;
	case 3: denom = 8; break;
	}

	ret = (2700000*N)/M;
	return (ret/denom)*10;
}
//@-node:decodePLL
//@+node:get_tm_freq
static int get_tm_freq(int tm,LUD_DEV* dev)
{
	/* defaults for TM1 */
	unsigned int clk_tm_reg = 0x47204;
	unsigned int clk_pll_reg    = 0x47004;

	unsigned int clk_tm,clk_mem;

	if (tm==2) {
		clk_tm_reg = 0x47208;
		clk_pll_reg = 0x47048;
	}

	clk_tm = MMIOread(dev,clk_tm_reg);

	if ((clk_tm&1) == 0) { /* clock not enabled */
		printk("TM%d: clock not enabled using default frequency\n",tm);
		return freq;
	}

	switch ((clk_tm>>1)&7) {
	case 0:	/* 27 MHz */
		return 27000000;
	case 1: /* PLL */
		return decodePLL(MMIOread(dev,clk_pll_reg));
	case 2: /* CLK MEM */
		clk_mem = MMIOread(dev,0x47100);
		if ((clk_mem&1)==0) { /* not enabled */
			printk("TM%d: clock_mem not enabled using default frequency\n",tm);
			return freq;
		}
		switch ((clk_mem>>1)&3) {
		case 0: /* 27 MHz */
			return 27000000;
		case 1: /* PLL */
			return decodePLL(MMIOread(dev,0x47014));
		case 2: /* 1.7 MHz */
			return 1700000;
		case 3: /* GPIO3 */
			printk("TM%d clkmem external clock - using default frequency\n",tm);
			return freq;
		}
		break;
	case 4:	/* GPIO4 */
		printk("TM%d uses external clock - using default frequency\n",tm);
		return freq;
	default:
		printk("TM%d wierd clock control reg - using default frequency\n",tm);
		return freq;
	}

	/* should never happen */
	return freq;
}
//@-node:get_tm_freq
//@+node:init_module


#ifdef CONFIG_PROC_FS
static void init_bw_mon(LUD_DEV* dev);
static void free_bw_mon(LUD_DEV* dev);
#endif

#if LINUX_VERSION_CODE >= 0x020600
static void trimedias_release(struct device* dev)
{
}
static struct platform_device trimedias = {
    .name = "trimedias",
    .id = -1,
};
#endif

int init_module(void)
{
	unsigned long mmio_base, sdram_base ;
	unsigned long sdram_size  ;

#if LINUX_VERSION_CODE >= 0x020600
	int ret = platform_device_register(&trimedias);
	if (ret) {
		return ret;
	}
	/* add a release function, to stop kernel panic when module is
	   removed */
	trimedias.dev.release = trimedias_release;
#endif


	mmio_base =  mmio;

#ifdef CONFIG_SYSFS
	viper_desc1.parent = &trimedias.dev;
#endif
	sdram_base = mips_size; /* 20050902 NAB */
	viper_desc1.mmio_log = remap_pci_mem(mmio_base, TM_MMIO_SIZE) ;
	viper_desc1.mmio_phy = mmio_base ;
	viper_desc1.mmio_offset = 0x40000;
	sdram_size = tm0_size; /* 20050902 NAB */
	viper_desc1.sdram_log  = remap_pci_mem(sdram_base, sdram_size) ;
	viper_desc1.sdram_phy  = sdram_base ;
	viper_desc1.sdram_size = sdram_size ;
	viper_desc1.freq = get_tm_freq(1,&viper_desc1);
	if (viper_desc1.freq != freq) {
		printk("TM0: detected clock %u\n",viper_desc1.freq);
	}
	viper_desc1.flags = 0;
	viper_desc1.irq = VIPER_TM1_IRQ;

	MMIOwrite(0x1e010007,(&viper_desc1),0x3e408);

	if (request_irq(viper_desc1.irq, viper_int_handler1,
				  0/*SA_SHIRQ*/, "pnx8550-1", &viper_desc1)) {
		printk("tm: Cannot grab IRQ : %d.\n", (int)viper_desc1.irq) ;
		return(-EIO) ;
	}

	viper_desc1.driver_module = THIS_MODULE;
	register_luddite_device(&viper_desc1);

#ifdef CONFIG_SYSFS
	viper_desc2.parent = &trimedias.dev;
#endif
	sdram_base = mips_size + tm0_size; /* 20050902 NAB */
	viper_desc2.mmio_log = remap_pci_mem(mmio_base, TM_MMIO_SIZE) ;
	viper_desc2.mmio_phy = mmio_base ;
	viper_desc2.mmio_offset = 0x60000;
	sdram_size = tm1_size; /* 20050902 NAB */
	viper_desc2.sdram_log  = remap_pci_mem(sdram_base, sdram_size) ;
	viper_desc2.sdram_phy  = sdram_base ;
	viper_desc2.sdram_size = sdram_size ;
	viper_desc2.freq = get_tm_freq(2,&viper_desc2);
	if (viper_desc2.freq != freq) {
		printk("TM1: detected clock %u\n",viper_desc2.freq);
	}
	viper_desc2.flags = 0;
	viper_desc2.irq = VIPER_TM2_IRQ;

	MMIOwrite(0x1e010007,(&viper_desc2),0x3e40c);

	if (request_irq(viper_desc2.irq, viper_int_handler2,
				  0/*SA_SHIRQ*/, "pnx8550-2", &viper_desc2)) {
		printk("tm: Cannot grab IRQ : %d.\n", (int)viper_desc2.irq) ;
		return(-EIO) ;
	}

	if (ndkbis) {
		viper_desc1.ndkbis =
		viper_desc2.ndkbis =
					/* 2005-05-10 Wouter Batelaan (WB) */
					remap_pci_mem(ram_size - NDK_BIS_SIZE,
					  NDK_BIS_SIZE);
	}

	viper_desc2.driver_module = THIS_MODULE;
	register_luddite_device(&viper_desc2);

#ifdef CONFIG_PROC_FS
	init_bw_mon(&viper_desc1);
#endif
	return 0;
}
//@-node:init_module
//@+node:cleanup_module

void cleanup_module(void)
{
#if LINUX_VERSION_CODE < 0x020600
	if (MOD_IN_USE) {
		return ;
	}
#endif

#ifdef CONFIG_PROC_FS
	free_bw_mon(&viper_desc1);
#endif
	unregister_luddite_device(&viper_desc1);
	free_irq(viper_desc1.irq, &viper_desc1) ;
	unregister_luddite_device(&viper_desc2);
	free_irq(viper_desc2.irq, &viper_desc2) ;

#if LINUX_VERSION_CODE >= 0x020600
	platform_device_unregister(&trimedias);
#endif
}
//@-node:cleanup_module
//@+node:bw_find_com

#ifdef CONFIG_PROC_FS

#define BW_MREG_INIT 0xff00

static int bw_help = 0;
static unsigned int bw_mreg[3] = {BW_MREG_INIT,BW_MREG_INIT,BW_MREG_INIT};

#define BW_0	0
#define BW_1	1
#define BW_2	2
#define BW_HELP	3
#define BW_CLEAR 4

struct BW_COMMS {
	char* command;
	int len;
	int val;
	int mask;
	int or;
} bw_comms[] = {
	{ "0", 1, 0,0,0 },
	{ "1", 1, 1,0,0 },
	{ "2", 1, 2,0,0 },
	{ "help", 4, 0,0,0 },
	{ "clear", 5, 0,0,0 },
	{ "r", 1, 0x1000000,0xf000000,0 },
	{ "w", 1, 0x2000000,0xf000000,0 },
	{ "rw", 2, 0x3000000,0xf000000,0 },
	{ "time", 4, 0x100000,0xf00000,0 },
	{ "number", 6, 0x200000,0xf00000,0 },
	{ "bw", 2, 0x300000,0xf00000,0 },
	{ "idle", 4, 0x400000,0xf00000,0 },
	{ "maxlat", 6, 0x500000,0xf00000,0 },
	{ "totlat", 6, 0x600000,0xf00000,0 },
	{ "mips", 4, 0x10000,0xf0000,1 },
	{ "tm1", 3, 0x20000,0xf0000,1 },
	{ "tm2", 3, 0x40000,0xf0000,1 },
	{ "hub", 3, 0x80000,0xf0000,1 },
	{ "msp1", 4, 0xe8,0xff,0 },
	{ "msp2", 4, 0xe4,0xff,0 },
	{ "tsdma", 5, 0xc8,0xff,0 },
	{ "vip1", 4, 0xc0,0xff,0 },
	{ "vip2", 4, 0xc4,0xff,0 },
	{ "dmag", 4, 0xb4,0xff,0 },
	{ "i2c", 3, 0xb0,0xff,0 },
	{ "sc", 2, 0xac,0xff,0 },
	{ "pci", 3, 0xa8,0xff,0 },
	{ "de", 2, 0xa4,0xff,0 },
	{ "usb", 3, 0xa0,0xff,0 },
	{ "tunnel", 6, 0x84,0xff,0 },
	{ "spdif", 5, 0x80,0xff,0 },
	{ "a1", 2, 0x81,0xff,0 },
	{ "a2", 2, 0x82,0xff,0 },
	{ "gpio", 4, 0x83,0xff,0 },
	{ "qvcp1", 5, 0x60,0xff,0 },
	{ "qvcp2", 5, 0x24,0xff,0 },
	{ "qtnr_r", 6, 0x44,0xff,0 },
	{ "qtnr_w", 6, 0x4c,0xff,0 },
	{ "mbs_r", 5, 0x40,0xff,0 },
	{ "mbs_w", 5, 0x48,0xff,0 },
	{ "vld", 3, 0x20,0xff,0 },
	{ "edma", 4, 0x08,0xff,0 },
	{ "dvd", 3, 0x04,0xff,0 },
	{ "mpeg", 4, 0x0,0xff,0 },
	{ NULL, 0 },
};

static int bw_find_com(const char* b,int* offset)
{
	int i = 0;
	int len = 0;
	struct BW_COMMS* bw = bw_comms;
	const char* p = b + *offset;
	const char* e;

	while (((*p=='\n')||(*p==' ')||(*p=='\t'))&&(*p!='\0')) p++;

	if (*p=='\0') return -1;

	e = p+1;
	while ((*e != '\n') && (*e!=' ')&&(*e!='\t')&&(*e!='\0')) e++;

	len = e-p;

	while (bw->command) {
		if (strncmp(bw->command,p,len)==0) {
			*offset = e-b;
			return i;
		}
		i++;
		bw++;
	}

	return -2;
}
//@-node:bw_find_com
//@+node:bw_read

static char* bw_help_message[] = {
	"help or 0,1,2 followed by commands to set the modes",
	" clear             resets the modes",
	" transaction type: one of r, w, rw,",
	" monitor modes:    one of time, number, bw, idle, maxlat, totlat",
	" mtl ports:        any combination of mips, tm1, tm2 and hub",
	" hub devices:      can be msp1,msp2,tsdma,vip1,vip2,dmag,i2c,",
	"                          sc,pci,de,usb,tunnel,spdif,a1,a2,gpio",
	"                          qvcp1,qvcp2,qtnr_r,qtnr_w,mbs_r,mbs_w",
	"			   vld,edma,dvd,mpeg",
	NULL
};

static int bw_read(char* page,char **start, off_t offset,
		int count,int* eof,void* data)
{

	int len = 0;
	LUD_DEV* dev = (LUD_DEV*)data;

	if (bw_help) {
		int i=0;
		len = 0;
		while (bw_help_message[i]!=NULL) {
			len += sprintf(page+len,"%s\n",bw_help_message[i]);
			i++;
		}
		bw_help = 0;
	} else {
		MMIOwrite(bw_mreg[0],dev,0x67000);
		MMIOwrite(bw_mreg[1],dev,0x67004);
		MMIOwrite(bw_mreg[2],dev,0x67008);

		MMIOwrite(0x888,dev,0x6700c); /* clear all counters */
		MMIOwrite(0x111,dev,0x6700c); /* start all counters */

		set_current_state(TASK_INTERRUPTIBLE);
		schedule_timeout(HZ/2); /* wait 1/2 a second */

		MMIOwrite(0x222,dev,0x6700c); /* stop all counters */

		len = sprintf(page,"0x%08x%08x 0x%08x%08x 0x%08x%08x\n",
			MMIOread(dev,0x67014),MMIOread(dev,0x67010),
			MMIOread(dev,0x6701c),MMIOread(dev,0x67018),
			MMIOread(dev,0x67024),MMIOread(dev,0x67020));
	}

	*eof = 1;
	return len;
}
//@-node:bw_read
//@+node:bw_write


static int bw_write(struct file *filp,
	const char *buffer,unsigned long count,void *data)
{
	if (filp->f_pos == 0) { /* first write */
		int com;
		int offset = 0;
		int index = 0;
		switch ((com=bw_find_com(buffer,&offset))) {
		case BW_HELP:
			bw_help = 1;
			break;
		case BW_0:
		case BW_1:
		case BW_2:
			index = com;
			break;
		default:
			return count;
		}
		com = bw_find_com(buffer,&offset);
		while (com != -1) {
			if (com == BW_CLEAR) {
				bw_mreg[index] = BW_MREG_INIT;
			} else {
				if (bw_comms[com].or) {
					bw_mreg[index] |= bw_comms[com].val;
				} else {
					bw_mreg[index] =
					 (bw_mreg[index]&~bw_comms[com].mask)
						| bw_comms[com].val;
				}
			}
			com = bw_find_com(buffer,&offset);
		}
	}

	return count;
}
//@-node:bw_write
//@+node:init_bw_mon

void init_bw_mon(LUD_DEV* dev)
{
	struct proc_dir_entry *bw;
	if (dev->proc_root) {
		bw = create_proc_entry("bw",0,dev->proc_root);
		bw->data = dev;
		bw->read_proc = bw_read;
		bw->write_proc = bw_write;
	}
}
//@-node:init_bw_mon
//@+node:free_bw_mon

void free_bw_mon(LUD_DEV* dev)
{
	if (dev->proc_root) {
		remove_proc_entry("bw",dev->proc_root);
	}
}
#endif /* CONFIG_PROCFS */
//@-node:free_bw_mon
//@-others
//@-node:@file host/lud-pnx8550.c
//@-leo
