/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Copyright (C) 2006 NXP B.V.
 * All Rights Reserved.
*//**
    @file   phStbProc_BaseProcSystem.c
    @brief  Base Proc Filesystem for modules to use.

@b  Component:  phStbProc.

    Linux Kernel module that creates the following proc system

    proc/
        nxp/
            drivers/
                pnx8550/
                    video/
                    audio/
                    media/
                    i2c/.

    This allows other modules to use this module to make sure that these entries
    exist.
*//*

Rev Date        Author      Comments
--------------------------------------------------------------------------------
001 15042005    laird       First Revision
002 20070730    laird       Header Update
--------------------------------------------------------------------------------
    For consistency and standardisation retain the Section Separators.
*/
/*******************
* INCLUDE FILES    *
********************/
#include <linux/proc_fs.h>

/**************************************************************
* EXPORTED MACROS         PH[<LNX|TM|NONE>]_IPSTBVIDEORENDERER_<_WORD>+        *
***************************************************************/

/**************************************************************
* EXPORTED TYPEDEFS       [p]ph[<lnx|tm|NONE>]StbVideoRenderer_<Word>+_t  *
***************************************************************/

/*******************************************************************
* EXPORTED DATA      g[k|p|kp|pk|kpk]ph[<lnx|tm|NONE>]StbVideoRenderer_<Word>+ *
********************************************************************/

/*******************************************************************
* EXPORTED FUNCTION PROTOTYPES     ph[<lnx|tm|NONE>]StbVideoRenderer[_<Word>+]   *
********************************************************************/

/******************************************************************************/
/** Get the drivers proc entry thas has been created.

    @param procEntry @b OUT Pointer to pointer of proc entry.

    @retval int
*/
int phStbProc_GetDriversProcEntry(struct proc_dir_entry **procEntry);

/******************************************************************************/
/** Get the Philips proc entry thas has been created.

    @param procEntry @b OUT Pointer to pointer of proc entry.

    @retval int
*/
int phStbProc_GetPhilipsProcEntry(struct proc_dir_entry **procEntry);

/******************************************************************************/
/** Get the PNX8550 proc entry thas has been created.

    @param procEntry @b OUT Pointer to pointer of proc entry.

    @retval int
*/
int phStbProc_GetPNX8550ProcEntry(struct proc_dir_entry **procEntry);

/******************************************************************************/
/** Get the I2C proc entry thas has been created.

    @param procEntry @b OUT Pointer to pointer of proc entry.

    @retval int
*/
int phStbProc_GetI2cProcEntry(struct proc_dir_entry **procEntry);

/******************************************************************************/
/** Get the audio proc entry thas has been created.

    @param procEntry @b OUT Pointer to pointer of proc entry.

    @retval int
*/
int phStbProc_GetAudioProcEntry(struct proc_dir_entry **procEntry);

/******************************************************************************/
/** Get the audio proc entry thas has been created.

    @param procEntry @b OUT Pointer to pointer of proc entry.

    @retval int
*/
int phStbProc_GetVideoProcEntry(struct proc_dir_entry **procEntry);

/******************************************************************************/
/** Get the audio proc entry thas has been created.

    @param procEntry @b OUT Pointer to pointer of proc entry.

    @retval int
*/
int phStbProc_GetMediaProcEntry(struct proc_dir_entry **procEntry);
